/* $Id$
 * env_lib.h: Environmental monitoring library code for 
 *            accessing I2C clients (chips) via i2c-envctrl 
 *            PCF8584 bus nexus interface 
 *
 * Copyright (C) 2004-2005 Eric Brower (ebrower@gmail.com)
 * 
 * TODO: functions should all return explicit error codes
 */
 
#ifndef _SPARC64_ENV_LIB_H
#define _SPARC64_ENV_LIB_H
 
#include <asm/io.h>
#include <linux/types.h>
#include <linux/i2c.h>

typedef enum envlib_flags {
	ENVLIB_FLAG_8591_NO_AOE = (1<<0), /* don't enable AOE on PCF8591 */
} envlib_flags_t;

/*
 * Library initialization --
 * This MUST be done prior to using any other library functions.
 *
 * @flags - platform-dependent values (if required)
 */
void env_lib_init(struct pcf_envctrl_nexus *nex, 
                  struct i2c_adapter *adap, 
                  envlib_flags_t flags);


/* 
 * "chip" access functions--
 * We use spinlocks for two reasons:
 *  1) because the i2c core doesn't support use while in_interrupt()
 *     so we "wrap" their semaphores with spinlocks (ugh!)
 *  2) to prevent concurrent access to bus from interrupts or timers
 *
 * Unlocked routines are provided for when multiple I2C message
 * atomicity is required-- the i2c core does not currently allow us
 * to issue multiple messages with dynamic content (i.e. read,xor,write).
 * When using unlocked functions (__xxx() funcs) you must acquire and
 * release the spinlock yourself-- this gives us a level of atomicity
 * but does not address multi-master concerns because all such transactions
 * are done as separate START/STOP sequences (FIXME in i2c core).
 */
 
/* returns 0 on success, error code on failure */
int __env2_i2c_readb(u8 addr, u8 *rv);
int env2_i2c_readb(u8 addr, u8 *rv);
 
int __env2_i2c_writeb(u8 addr, u8 val);
int env2_i2c_writeb(u8 addr, u8 val);
 
/* XXX Temporary-- should be lib code or somesuch */
#define __envctrl_i2c_write_8574(addr,data) __env2_i2c_writeb(addr,data);
#define envctrl_i2c_write_8574(addr,data) env2_i2c_writeb(addr,data);
 
u8 __envctrl_i2c_read_8574(u8 addr);
u8 envctrl_i2c_read_8574(u8 addr);
 
int __envctrl_i2c_read_8591(u8 addr, u8 port, u8 *rv);
int envctrl_i2c_read_8591(u8 addr, u8 port, u8 *rv);
 
u8 __envctrl_i2c_write_8591(u8 addr, u8 data);
u8 envctrl_i2c_write_8591(u8 addr, u8 data);
 
u8 __envctrl_i2c_read_8570(u8 addr, u8 *buf, u8 byteno);
u8 envctrl_i2c_read_8570(u8 addr, u8 *buf, u8 byteno);
 
 
#define ENVLIB_TDA8444_PORT_MAX (7)	/* 0 - 7 inclusive  */
#define ENVLIB_TDA8444_DATAMSK  (0x3f)  /* low 6 bits only  */
int __envctrl_i2c_read_8444(u8 addr, u8 port, u8 *rv);
int envctrl_i2c_read_8444(u8 addr, u8 port, u8 *rv);
 
int __envctrl_i2c_write_8444(u8 addr, u8 port, u8 data);
int envctrl_i2c_write_8444(u8 addr, u8 port, u8 data);
 
void env_lib_scan_bus(u8 start, u8 end, int read);

#define ENV_LIB_LM75_REG_TEMP 	(0x00)
#define ENV_LIB_LM75_REG_CFG	(0x01)
#define ENV_LIB_LM75_REG_THYST	(0x02)
#define ENV_LIB_LM75_REG_TOS	(0x03)
#define ENV_LIB_LM75_REG_MSK	(0x03)

#define ENV_LIB_LM75_CFG_SHTDN	(1<<0)
#define ENV_LIB_LM75_CFG_INT	(1<<1) /* intr mode, comparison if unset     */
#define ENV_LIB_LM75_CFG_POLHI	(1<<2) /* OS polarity HI, LO if unset        */
#define ENV_LIB_LM75_CFG_FQ2	(1<<3) /* fault queue depth +2 if set */
#define ENV_LIB_LM75_CFG_FQ4	(1<<4) /* fault queue depth +4 if set */
#define ENV_LIB_LM75_CFG_MSK    (0x1F) /* available bits (don't touch 5,6,7) */

int envctrl_i2c_read_LM75(u8 addr, u8 reg, u16 *rv);
int envctrl_i2c_read_LM75_temp(u8 addr, u8 *rv);

int envctrl_i2c_write_LM75(u8 addr, u8 reg, u16 val);

#define ENV_LIB_8583_REG_MSK		(0x0F)
#define ENV_LIB_8583_REG_CSR		(0x00)
#define ENV_LIB_8583_REG_TIMER		(0x07)
#define ENV_LIB_8583_REG_ALRMCTL	(0x08)
#define ENV_LIB_8583_REG_ALRMTMR	(0x0F)

int __envctrl_i2c_read_pcf8583(u8 addr, u8 reg, u8 *rv);
int envctrl_i2c_read_pcf8583(u8 addr, u8 port, u8 *rv);
int __envctrl_i2c_write_pcf8583(u8 addr, u8 reg, u8 data);
int envctrl_i2c_write_pcf8583(u8 addr, u8 reg, u8 data);
 
#endif /* #ifndef _SPARC64_ENV_LIB_H */
